@@ -11,14 +11,17 @@ else
     BLAZAR_BIN_DIR=$(get_python_exec_prefix)
 fi
 
-# Test if any Ceilometer services are enabled
-# is_ceilometer_enabled
+# These packages should be tested under Python 3 when enabled by DevStack
+enable_python3_package blazar blazar-nova python-blazarclient
+
+# Test if any Blazar services are enabled
+# is_blazar_enabled
 function is_blazar_enabled {
     [[ ,${ENABLED_SERVICES} =~ ,"blazar-" ]] && return 0
     return 1
 }
 
-# Oslo.Messaging RPC iniupdate cofiguration
+# Oslo.Messaging RPC iniupdate configuration
 function iniupdate_rpc_backend {
     local file=$1
     local section=$2
@@ -38,27 +41,25 @@ function configure_blazar {
     fi
     sudo chown $STACK_USER $BLAZAR_CONF_DIR
 
-    BLAZAR_POLICY_FILE=$BLAZAR_CONF_DIR/policy.json
-    cp $BLAZAR_DIR/etc/policy.json $BLAZAR_POLICY_FILE
-
     touch $BLAZAR_CONF_FILE
 
     iniset $BLAZAR_CONF_FILE DEFAULT os_auth_version v3
+    iniset $BLAZAR_CONF_FILE DEFAULT os_auth_host $KEYSTONE_AUTH_HOST
     iniset $BLAZAR_CONF_FILE DEFAULT os_auth_port 80
     iniset $BLAZAR_CONF_FILE DEFAULT os_auth_prefix identity
     iniset $BLAZAR_CONF_FILE DEFAULT os_admin_password $SERVICE_PASSWORD
     iniset $BLAZAR_CONF_FILE DEFAULT os_admin_username blazar
     iniset $BLAZAR_CONF_FILE DEFAULT os_admin_project_name $SERVICE_TENANT_NAME
     iniset $BLAZAR_CONF_FILE DEFAULT identity_service $BLAZAR_IDENTITY_SERVICE_NAME
+    iniset $BLAZAR_CONF_FILE DEFAULT os_region_name $REGION_NAME
 
-    # keystone authtoken
+    # Keystone authtoken
     _blazar_setup_keystone $BLAZAR_CONF_FILE keystone_authtoken
 
-    iniset $BLAZAR_CONF_FILE physical:host aggregate_freepool_name $BLAZAR_FREEPOOL_NAME
+    iniset $BLAZAR_CONF_FILE nova aggregate_freepool_name $BLAZAR_FREEPOOL_NAME
 
     iniset $BLAZAR_CONF_FILE DEFAULT host $HOST_IP
     iniset $BLAZAR_CONF_FILE DEFAULT debug $BLAZAR_DEBUG
-    iniset $BLAZAR_CONF_FILE DEFAULT verbose $BLAZAR_VERBOSE
 
     iniset $BLAZAR_CONF_FILE manager plugins physical.host.plugin,virtual.instance.plugin
 
@@ -76,18 +77,18 @@ function configure_blazar {
         iniadd $NOVA_CONF filter_scheduler enabled_filters "RetryFilter, AvailabilityZoneFilter, RamFilter, ComputeFilter, ComputeCapabilitiesFilter, ImagePropertiesFilter, AggregateInstanceExtraSpecsFilter, AggregateMultiTenancyIsolation, ServerGroupAntiAffinityFilter, ServerGroupAffinityFilter, BlazarFilter"
     else
         if [[ $ACTUAL_FILTERS != *AggregateInstanceExtraSpecsFilter* ]];  then
-            iniset $NOVA_CONF filter_scheduler enabled_filters "$ACTUAL_FILTERS,AggregateInstanceExtraSpecsFilter"
+	    ACTUAL_FILTERS="$ACTUAL_FILTERS,AggregateInstanceExtraSpecsFilter"
         fi
         if [[ $ACTUAL_FILTERS != *AggregateMultiTenancyIsolation* ]];  then
-            iniset $NOVA_CONF filter_scheduler enabled_filters "$ACTUAL_FILTERS,AggregateMultiTenancyIsolation"
+	    ACTUAL_FILTERS="$ACTUAL_FILTERS,AggregateMultiTenancyIsolation"
         fi
         if [[ $ACTUAL_FILTERS != *ServerGroupAntiAffinityFilter* ]];  then
-            iniset $NOVA_CONF filter_scheduler enabled_filters "$ACTUAL_FILTERS,ServerGroupAntiAffinityFilter"
+	    ACTUAL_FILTERS="$ACTUAL_FILTERS,ServerGroupAntiAffinityFilter"
         fi
         if [[ $ACTUAL_FILTERS != *BlazarFilter* ]];  then
-            iniset $NOVA_CONF filter_scheduler enabled_filters "$ACTUAL_FILTERS,BlazarFilter"
+	    ACTUAL_FILTERS="$ACTUAL_FILTERS,BlazarFilter"
         fi
-
+	iniset $NOVA_CONF filter_scheduler enabled_filters $ACTUAL_FILTERS
     fi
 
     ACTUAL_AVAILABLE_FILTERS=$(iniget $NOVA_CONF filter_scheduler available_filters)
@@ -96,9 +97,9 @@ function configure_blazar {
     fi
     iniadd $NOVA_CONF filter_scheduler available_filters "blazarnova.scheduler.filters.blazar_filter.BlazarFilter"
 
-    # TODO(hiro-kobayashi): This line should be deleted after the patch
-    # https://review.openstack.org/#/c/471590/ is merged.
-    iniadd $NOVA_CONF blazar:physical:host blazar_az_prefix blazar_
+    if [[ "$BLAZAR_USE_MOD_WSGI" == "True" ]]; then
+        write_uwsgi_config "$BLAZAR_UWSGI_CONF" "$BLAZAR_UWSGI" "/reservation"
+    fi
 
     # Database
     recreate_database blazar utf8
@@ -107,7 +108,7 @@ function configure_blazar {
     $BLAZAR_BIN_DIR/blazar-db-manage --config-file $BLAZAR_CONF_FILE upgrade head
 }
 
-# Configures keystone integration for blazar service
+# Configures Keystone integration for the Blazar service
 function _blazar_setup_keystone {
     local conf_file=$1
     local section=$2
@@ -132,14 +133,14 @@ function create_blazar_cache_dir {
 
 # create_blazar_aggregate_freepool() - Create a Nova aggregate to use as freepool (for host reservation)
 function create_blazar_aggregate_freepool {
-    openstack aggregate create $BLAZAR_FREEPOOL_NAME
+    openstack --os-region-name $REGION_NAME aggregate create $BLAZAR_FREEPOOL_NAME
 }
 
-# create_blazar_accounts() - Set up common required BLAZAR accounts
+# create_blazar_accounts() - Set up common required Blazar accounts
 #
 # Tenant               User       Roles
 # ------------------------------------------------------------------
-# service              BLAZAR     admin        # if enabled
+# service              blazar     admin        # if enabled
 #
 function create_blazar_accounts {
     SERVICE_TENANT=$(openstack project list | awk "/ $SERVICE_TENANT_NAME / { print \$2 }")
@@ -149,13 +150,17 @@ function create_blazar_accounts {
         "$SERVICE_PASSWORD" "default" "blazar@example.com")
     get_or_add_user_project_role $ADMIN_ROLE $BLAZAR_USER_ID $SERVICE_TENANT
 
+    if [[ "$BLAZAR_USE_MOD_WSGI" == "True" ]]; then
+        blazar_api_url="$BLAZAR_SERVICE_PROTOCOL://$BLAZAR_SERVICE_HOST/reservation"
+    else
+        blazar_api_url="$BLAZAR_SERVICE_PROTOCOL://$BLAZAR_SERVICE_HOST:$BLAZAR_SERVICE_PORT"
+    fi
+
     BLAZAR_SERVICE=$(get_or_create_service "blazar" \
-        "reservation" "Blazar Reservations Service")
+        "reservation" "Blazar Reservation Service")
     get_or_create_endpoint $BLAZAR_SERVICE \
         "$REGION_NAME" \
-        "$BLAZAR_SERVICE_PROTOCOL://$BLAZAR_SERVICE_HOST:$BLAZAR_SERVICE_PORT/v1" \
-        "$BLAZAR_SERVICE_PROTOCOL://$BLAZAR_SERVICE_HOST:$BLAZAR_SERVICE_PORT/v1" \
-        "$BLAZAR_SERVICE_PROTOCOL://$BLAZAR_SERVICE_HOST:$BLAZAR_SERVICE_PORT/v1"
+        "$blazar_api_url/v1"
 
     KEYSTONEV3_SERVICE=$(get_or_create_service "keystonev3" \
         "identityv3" "Keystone Identity Service V3")
@@ -180,9 +185,34 @@ function install_blazar {
 }
 
 
+# install_blazar_dashboard() - Install Blazar dashboard for Horizon
+function install_blazar_dashboard {
+    git_clone $BLAZAR_DASHBOARD_REPO $BLAZAR_DASHBOARD_DIR $BLAZAR_DASHBOARD_BRANCH
+    setup_develop $BLAZAR_DASHBOARD_DIR
+    blazar_setup_horizon
+}
+
+
+# Set up Horizon integration with Blazar
+function blazar_setup_horizon {
+    # Link Dashboard panel to Horizon's directory
+    ln -fs $BLAZAR_DASHBOARD_DIR/blazar_dashboard/enabled/_90_admin_reservation_panelgroup.py $HORIZON_DIR/openstack_dashboard/local/enabled/
+    ln -fs $BLAZAR_DASHBOARD_DIR/blazar_dashboard/enabled/_90_project_reservations_panelgroup.py $HORIZON_DIR/openstack_dashboard/local/enabled/
+    ln -fs $BLAZAR_DASHBOARD_DIR/blazar_dashboard/enabled/_91_admin_reservation_hosts_panel.py $HORIZON_DIR/openstack_dashboard/local/enabled/
+    ln -fs $BLAZAR_DASHBOARD_DIR/blazar_dashboard/enabled/_91_project_reservations_leases_panel.py $HORIZON_DIR/openstack_dashboard/local/enabled/
+
+    # Restart Horizon
+    restart_apache_server
+}
+
+
 # start_blazar() - Start running processes, including screen
 function start_blazar {
-    run_process blazar-a "$BLAZAR_BIN_DIR/blazar-api --debug --config-file $BLAZAR_CONF_FILE"
+    if [ "$BLAZAR_USE_MOD_WSGI" == "True" ]; then
+        run_process 'blazar-a' "$BLAZAR_BIN_DIR/uwsgi --ini $BLAZAR_UWSGI_CONF"
+    else
+        run_process blazar-a "$BLAZAR_BIN_DIR/blazar-api --debug --config-file $BLAZAR_CONF_FILE"
+    fi
     run_process blazar-m "$BLAZAR_BIN_DIR/blazar-manager --debug --config-file $BLAZAR_CONF_FILE"
 }
 
@@ -195,14 +225,27 @@ function stop_blazar {
 }
 
 
+# clean_blazar_configuration() - Cleanup blazar configurations
+function clean_blazar_configuration {
+    if [[ "$BLAZAR_USE_MOD_WSGI" == "True" ]]; then
+        remove_uwsgi_config "$BLAZAR_UWSGI_CONF" "$BLAZAR_UWSGI"
+    fi
+}
+
+
 if is_service_enabled blazar blazar-m blazar-a; then
     if [[ "$1" == "stack" && "$2" == "pre-config" ]]; then
         echo "Pre installation steps for Blazar"
         iniupdate_rpc_backend
     elif [[ "$1" == "stack" && "$2" == "install" ]]; then
         echo_summary "Installing Blazar"
-        # Use stack_install_service here to account for vitualenv
+        # Use stack_install_service here to account for virtualenv
         stack_install_service blazar
+
+        if is_service_enabled horizon; then
+            install_blazar_dashboard
+        fi
+
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         echo_summary "Configuring Blazar"
         configure_blazar
@@ -217,6 +260,7 @@ if is_service_enabled blazar blazar-m blazar-a; then
     if [[ "$1" == "unstack" ]]; then
         echo_summary "Shutting Down Blazar"
         stop_blazar
+        clean_blazar_configuration
     fi
 
 fi
